#!/usr/bin/python3

from tkinter import *
from tkinter import messagebox
from tkinter import ttk
from tkinter import font

import afrikalan.tkinter
import afrikalan.manual

from PIL import Image, ImageTk
from lxml import etree
from pprint import pprint
import os
import sys
import textwrap

import math

import argparse


glob_dialog_bg="#FFFFFF"
glob_dialog_padx=24
glob_dialog_pady=10
glob_infoframe_bg="#FFFFFF"
glob_button_bg="#E0E0E0" #was: #D4D4D4


#####################################
# Partie relative à l'interface
#####################################


class Animators:
  def __init__(self, contentHandler, xmlAnims, canvas, offsetX=0, offsetY=0):
    self.contentHandler=contentHandler
    self.xmlAnims=xmlAnims
    self.canvas=canvas
    self.offsetX=offsetX
    self.offsetY=offsetY
    self.animators=[]
    self.createAnimators()
    self.syncDurations()

  def createAnimators(self):
    for nodeAnim in self.xmlAnims:
      self.animators.append(Animator(self.contentHandler, nodeAnim, self.canvas, offsetX=self.offsetX, offsetY=self.offsetY))
      
  def syncDurations(self):
    maxduration=0
    for animator in self.animators: 
      maxduration=max(maxduration, animator.fullDuration)
      
    for animator in self.animators: 
      animator.padToDuration(maxduration)
    

  def finish(self):
    for animator in self.animators:
      animator.finish()

  def update(self, time):
    for animator in self.animators:
      animator.update(time)

#x = x0 + r*cos(t)
#y = y0 + r*sin(t)

class Animator:
  def __init__(self, contentHandler, xmlAnim, canvas, offsetX=0, offsetY=0):
    self.contentHandler=contentHandler
    self.xmlAnim=xmlAnim
    self.canvas=canvas
    self.steps=[]
    self.fullDuration=0
    self.offsetX=offsetX
    self.offsetY=offsetY
    #self.defaultColor="#f57900"
    self.defaultColor="#599de3"

    self.lastProcessedStepId=-1
    self.analyseSteps()
    self.arEltIdInCanvas=[]
    self.photoImages={}
  
    
  def analyseSteps(self):
    actOffsetStart=0
    for nodeStep in self.xmlAnim:
      duration=int(nodeStep.attrib.get('duration', 3000))
      blink=nodeStep.attrib.get('blink', False)
      if blink is False:
        self.steps.append({'start': actOffsetStart, 'end': actOffsetStart+duration, 'node': nodeStep})
        actOffsetStart+=duration
      else:
        blinkDuration=int(blink)
        blinkStep=0
        blinkTimeOffset=0
        while blinkTimeOffset<duration:
          if blinkStep%2==0:
            self.steps.append({'start': actOffsetStart+blinkTimeOffset, 'end': actOffsetStart+blinkTimeOffset+blinkDuration, 'node': nodeStep})
          else:
            self.steps.append({'start': actOffsetStart+blinkTimeOffset, 'end': actOffsetStart+blinkTimeOffset+blinkDuration, 'node': False})
          blinkStep+=1
          blinkTimeOffset+=blinkDuration
        actOffsetStart+=blinkTimeOffset
      
    self.fullDuration=actOffsetStart
  
  def padToDuration(self, targetDuration): #keep all animators in sync
    if self.fullDuration<targetDuration:
      self.steps.append({'start': self.fullDuration, 'end': targetDuration, 'node': False})
      self.fullDuration=targetDuration


  def getNodeId(self, timeOffset):
    stepId=0
    for step in self.steps:
      if timeOffset>=step['start'] and timeOffset<step['end']:
        return(stepId)
      stepId+=1
    raise NameError("Step not found!") #If nothing found...
      
    
  def finish(self):
    self.removeElt()

  def update(self, time):
    timeOffset=time%self.fullDuration
    actStepNodeId=self.getNodeId(timeOffset)
    if actStepNodeId!=self.lastProcessedStepId:
      self.removeElt()
        
      eltXmlNode=self.steps[actStepNodeId]['node']
      if eltXmlNode is not False:
        self.drawElt(eltXmlNode)
    
      self.lastProcessedStepId=actStepNodeId

  def removeElt(self):
    for eltId in self.arEltIdInCanvas:
      self.canvas.delete(eltId)
    
  
  def drawElt(self, eltXmlNode):
    x=int(eltXmlNode.attrib.get('x', '0'))+self.offsetX
    y=int(eltXmlNode.attrib.get('y', '0'))+self.offsetY
    w=int(eltXmlNode.attrib.get('w', '48'))
    h=int(eltXmlNode.attrib.get('h', '48'))
    x2=int(eltXmlNode.attrib.get('x2', '10'))+self.offsetX
    y2=int(eltXmlNode.attrib.get('y2', '10'))+self.offsetY
    outlineWidth=int(eltXmlNode.attrib.get('line-width', '3'))
    outlineColor=eltXmlNode.attrib.get('line-color', self.defaultColor)
    fillColor=eltXmlNode.attrib.get('fill', '') #Empty by default
    arrowHeadLen=int(eltXmlNode.attrib.get('arrow-len', '15'))
    arrowFillHead=int(eltXmlNode.attrib.get('arrow-fill-head', '1'))
    
    if eltXmlNode.tag=="oval":
      self.arEltIdInCanvas.append(self.canvas.create_oval(x, y, x+w, y+h, outline=outlineColor, width=outlineWidth, fill=fillColor))
    elif eltXmlNode.tag=="rectangle":
      self.arEltIdInCanvas.append(self.canvas.create_rectangle(x, y, x+w, y+h, outline=outlineColor, width=outlineWidth, fill=fillColor))
    elif eltXmlNode.tag=="text":
      textColor=eltXmlNode.attrib.get('text-color', self.defaultColor)
      textFont=eltXmlNode.attrib.get('text-font', 'serif 16 bold')
      textAnchor=eltXmlNode.attrib.get('text-anchor', 'center')
      textWrap=int(eltXmlNode.attrib.get('text-wrap', 0))
      textContent=eltXmlNode.text
      if textWrap != 0:
        textContent="\n".join(textwrap.wrap(textContent, textWrap))
      self.arEltIdInCanvas.append(self.canvas.create_text(x, y, fill=textColor, text=textContent, font=textFont, anchor=textAnchor))
    elif eltXmlNode.tag=="line":
      self.arEltIdInCanvas.append(self.canvas.create_line(x, y, x2, y2, fill=outlineColor, width=outlineWidth))
    elif eltXmlNode.tag=="arrow":
      self.drawArrow(x, y, x2, y2, outlineColor, outlineWidth, arrowHeadLen, arrowFillHead)
    elif eltXmlNode.tag=="cursor":
      cursorColor=eltXmlNode.attrib.get('color', "#000000")
      self.drawArrow(x+6, y+21, x, y, outlineColor=cursorColor, outlineWidth=outlineWidth, arrowHeadLen=14, arrowFillHead=1, arrowHeadAngle=50)
    elif eltXmlNode.tag=="image":
      imgPath=eltXmlNode.attrib.get('path', '')
      if self.photoImages.get(imgPath, False) is False:
        self.photoImages[imgPath]=ImageTk.PhotoImage(file = self.contentHandler.getDependencyPath(imgPath))
      self.arEltIdInCanvas.append(self.canvas.create_image(x, y, image=self.photoImages.get(imgPath), anchor="nw"))
    elif eltXmlNode.tag=="pause":
      pass
    else:
      raise NameError("unknown draw elt tag: "+eltXmlNode.tag)

  def drawArrow(self, x, y, x2, y2, outlineColor, outlineWidth, arrowHeadLen, arrowFillHead, arrowHeadAngle=66):
    self.arEltIdInCanvas.append(self.canvas.create_line(x, y, x2, y2, fill=outlineColor, width=outlineWidth))
    lx=x2-x
    ly=y2-y
    l=math.sqrt(lx*lx+ly*ly)
    angle=math.acos(lx/l)
    if y2<y:
      angle=-angle
    deltangle=arrowHeadAngle/2*math.pi/180
    a=[x2+math.cos(math.pi+angle+deltangle)*arrowHeadLen, y2+math.sin(math.pi+angle+deltangle)*arrowHeadLen]
    b=[x2+math.cos(math.pi+angle-deltangle)*arrowHeadLen, y2+math.sin(math.pi+angle-deltangle)*arrowHeadLen]
    if arrowFillHead==1:
      self.arEltIdInCanvas.append(self.canvas.create_polygon(x2, y2, a[0], a[1], b[0], b[1], fill=outlineColor, width=0))
    else:
      self.arEltIdInCanvas.append(self.canvas.create_line(x2, y2, a[0], a[1], fill=outlineColor, width=outlineWidth))
      self.arEltIdInCanvas.append(self.canvas.create_line(x2, y2, b[0], b[1], fill=outlineColor, width=outlineWidth))



#
# Interfaces 
#

class Interface_AffPages(Frame):
  
  def __init__(self, fenetre, contentHandler):
    self.fenetre=fenetre
    self.hasBeenCentered=False
    self.fond=glob_dialog_bg
    
    self.contentHandler=contentHandler
    self.pages=self.contentHandler.getPages()
    self.imageWidh, self.imageHeight=self.contentHandler.getImagesDim()
    self.textWidth=self.imageWidh
    self.instFontSize=14
    self.hintFontSize=12
    self.imageOffsetX=0
    self.imageOffsetY=0
    self.initImgs()
    self.animators=False
    
    self.animTickDelay=100
    self.animTimeStamp=0
    
    self.delaiFocusOut=4000
    self.jobFocusOut=False

    self.actPage=contentHandler.findStartPage()
    
    Frame.__init__(self, fenetre, padx=glob_dialog_padx, pady=glob_dialog_pady, bg=self.fond)
    self.pack(fill=BOTH)
    self.buildWidgets()
    
    self.fenetre.bind("<FocusOut>", self.evtFocusOut)
    self.fenetre.bind("<FocusIn>", self.evtFocusIn)
    
    self.wrapLenInst=int(self.textWidth/(self.instFontSize/1.7))
    self.wrapLenHint=int(self.textWidth/(self.hintFontSize/1.60))
    
    self.majAff()

  def getPageAffNum(self):
    return(self.actPage+1) #0-indexed to 1-indexed

  def buildWidgets(self):
    #self.titreGeneral = Label(self, text=self.contentHandler.getTitle(), bg=self.fond, font='Helvetica 10')
    #self.titreGeneral.grid(row=0, columnspan=4)

    self.contentFrame=Frame(self, bg=self.fond)
    self.contentFrame.pack()


    self.imgCanvas=Canvas(self.contentFrame, bg=self.fond, borderwidth=0, highlightthickness=0, width=self.imageWidh, height=self.imageHeight)
    self.imgCanvas.pack(side=RIGHT, fill=BOTH)
    self.imgCanvasImg=self.imgCanvas.create_image(0, 0, anchor="nw")

    Frame(self.contentFrame, width=16, bg=self.fond).pack(side=RIGHT) #H-Separator
    
    self.textFrame=Frame(self.contentFrame, bg=self.fond, width=self.textWidth, height=self.imageHeight)
    self.textFrame.pack_propagate(False)
    self.textFrame.pack(side=LEFT, fill=BOTH, ipadx=10)

    Frame(self.textFrame, height=4, bg=self.fond).pack() #V-Separator
    self.titre = Label(self.textFrame, text=self.contentHandler.getTitle(), pady=8, bg=self.fond, font='Helvetica 20 bold')
    self.titre.pack(pady=4, anchor=W)
    
    self.instructionHintContainer=Frame(self.textFrame, bg=self.fond, width=self.textWidth)
    self.instructionHintContainer.pack(anchor=W)
    self.instructionHintFrame=Frame(self.instructionHintContainer, bg=self.fond, width=self.textWidth)
    self.instructionHintFrame.pack(anchor=W)
    
    self.navBar=Frame(self.textFrame, bg=self.fond)
    self.navBar.pack(fill=X, pady=16) #side=BOTTOM, 
    self.bouton_prec = afrikalan.tkinter.Link(self.navBar, text="<- Retour", compound="left", bg=self.fond, fg="#599de3", font='Helvetica 20', command=self.evtPagePrec)
    self.bouton_prec.pack(side=LEFT)
    self.bouton_suiv = afrikalan.tkinter.Link(self.navBar, text="", compound="right", bg=self.fond, fg="#599de3", font='Helvetica 20', command=self.evtPageSuiv)
    self.bouton_suiv.pack(side=RIGHT)

    Frame(self, height=16, bg=self.fond).pack() #V-Separator
    
    self.icone_bouton_quitter=ImageTk.PhotoImage(file = r"/usr/share/afrikalan/images/transfert/icone_bouton_fermer.png")
    self.bouton_quitter = Button(self, text="Fermer", image=self.icone_bouton_quitter, compound="left", command=self.btnQuit, bg=glob_button_bg)
    self.bouton_quitter.pack(anchor=E)  

    self.fenetre.after(self.animTickDelay, self.tickAnim)


    
  def initImgs(self):
    self.photoimages=[]
    for page in self.pages:
      imgFile=page.get("image", "")
      if imgFile!="":
        self.photoimages.append(ImageTk.PhotoImage(file = imgFile))
      else:
        self.photoimages.append('') #valeur à utiliser d'après https://stackoverflow.com/questions/33302779/python-tkinter-remove-delete-image-from-label
  
  def evtPagePrec(self, evt=False):
    self.actPage-=1
    self.majAff()

  def evtPageSuiv(self, evt=False):
    if self.actPage>=len(self.pages)-1:
      self.quit()
    else:
      self.actPage+=1
      self.majAff()


  def evtFocusOutLong(self):
    if self.contentHandler.isQuitOnFocusOut():
      self.quit()

  def evtFocusIn(self, evt):
    if self.jobFocusOut is not False:
      self.after_cancel(self.jobFocusOut)
      self.jobFocusOut=False

  def evtFocusOut(self, evt):
    if self.jobFocusOut is False:
      self.jobFocusOut=self.after(self.delaiFocusOut, self.evtFocusOutLong)

  def majAff(self):
    dictPage=self.pages[self.actPage]
    
    self.titre.configure(text=dictPage.get("title", ""))
    
    
    self.instructionHintFrame.destroy()
    self.instructionHintFrame=Frame(self.instructionHintContainer, bg=self.fond, width=self.textWidth)
    self.instructionHintFrame.pack(anchor=W)
    
    for instruction in dictPage.get("instructions", []):
      arlinesInst=textwrap.wrap(instruction, self.wrapLenInst)
      Label(self.instructionHintFrame, text="\n".join(arlinesInst), pady=4, bg=self.fond, font='Helvetica '+str(self.instFontSize), justify="left").pack(anchor=W)
    
    Frame(self.instructionHintFrame, height=4, bg=self.fond).pack() #V-Separator
    
    for hint in dictPage.get("hints", []):
      arlinesHint=textwrap.wrap(hint, self.wrapLenHint)
      Label(self.instructionHintFrame, text="\n".join(arlinesHint), pady=2, bg=self.fond, font='Helvetica '+str(self.hintFontSize)+' italic', justify="left", fg="#777777").pack(anchor=W)
    
    
    
    
    #self.image.configure(image=self.photoimages[self.actPage])
    self.imageOffsetX=int((self.imageWidh-self.photoimages[self.actPage].width())/2)
    self.imageOffsetY=int((self.imageHeight-self.photoimages[self.actPage].height())/2)
    self.imgCanvas.coords(self.imgCanvasImg, self.imageOffsetX, self.imageOffsetY)
    self.imgCanvas.itemconfig(self.imgCanvasImg, image=self.photoimages[self.actPage])
    
    if self.actPage<=0:
      self.bouton_prec['state'] = 'disabled'
    else:
      self.bouton_prec['state'] = 'normal'
      
    if self.actPage>=len(self.pages)-1:
      self.bouton_suiv.configure(text="Terminé")
    else:
      self.bouton_suiv.configure(text=self.contentHandler.getNextLinkText())
    
    if self.animators is not False:
      self.animators.finish()
    
    self.animTimeStamp=0
    self.animators=Animators(contentHandler=self.contentHandler, xmlAnims=self.contentHandler.getAnims(self.actPage), canvas=self.imgCanvas, offsetX=self.imageOffsetX, offsetY=self.imageOffsetY)
    
    if self.hasBeenCentered is False:
      afrikalan.tkinter.center(self.fenetre)
      self.hasBeenCentered=True
      
      
  def btnQuit(self):
    if self.animators is not False:
      self.animators.finish()
    super().quit()

  def tickAnim(self):
    self.animTimeStamp+=self.animTickDelay
    
    if self.animators is not False:
      self.animators.update(self.animTimeStamp)
    
    self.fenetre.after(self.animTickDelay, self.tickAnim)
    


##############################################
# Partie relative au pilotage de l'appli ... #
##############################################

  

class Logger():
  def __init__(self, contentPath='NA', callFrom="NA", ):
    self.contentPath=contentPath
    self.callFrom=callFrom
    
  def log(self, event, pageNum="--"):
    logstr='('+self.callFrom+')'
    logstr+=self.contentPath
    logstr+='{'+str(pageNum)+'}'
    logstr+=' ### '+event
    
    logfile=open("/rw/affMan.log", "a")
    logfile.write(logstr+'\n')
    logfile.close()

def parseArgs():
  argparser=argparse.ArgumentParser(description='Afficheur d\'aide Afrikalan')
  argparser.add_argument("contentPath", help="chemin vers un fichier XML contenant le guide à afficher. l'ensemble des dépendances (images, etc...) doit se trouver dans le même répertoire que le fichier.")
  argparser.add_argument("--from", dest="callfrom", help="application ayant initié l'affichage du manuel (utile pour les stats)", default="N/A")
  argparser.add_argument("--staytop", choices=['y', 'n'], dest="staytop", help="Si définir à y, la fenetre restera en haut de l'affichage", default="n")
  return(argparser.parse_args())

  
  
#######################
# Main part
#######################

def affErr(exctype, value, tb):
  messagebox.showerror(message="Toutes nos excuses, une erreur s'est produite:\n\n"+str(value))

args=parseArgs()
content=afrikalan.manual.ContentHandler(args.contentPath, args.callfrom)
#pprint(content.getPages())
#sys.exit(0)

logger=Logger(contentPath=args.contentPath, callFrom=args.callfrom)

root=Tk()
if content.isWinStayOnTop() or args.staytop=="y":
  root.attributes('-topmost', True)
root.title('Guide Afrikalan: '+content.getTitle())


ifAffPages = Interface_AffPages(root, content)
logger.log("DEBUT", ifAffPages.getPageAffNum())
ifAffPages.mainloop()
ifAffPages.destroy()
logger.log("FIN", ifAffPages.getPageAffNum())

  
