#!/usr/bin/python3


from tkinter import *
import tkinter.messagebox
from PIL import Image, ImageTk
import subprocess
from pprint import pprint
import threading
import os
import sys
import urllib3.request
import argparse
import time
import afrikalan.tkinter
import textwrap
import psutil

#vnc-in.afrikalan.org

glob_dialog_bg="#F8F8F8"
glob_dialog_padx=24
glob_dialog_pady=10
glob_infoframe_bg="#FFFFFF"
glob_button_bg="#E0E0E0" #was: #D4D4D4


def wrap(txt, wrplen):
  return("\n".join(textwrap.wrap(txt, wrplen)))

class Interface_Assistance(Frame):
  
  def __init__(self, fenetre):
    self.fenetre=fenetre
    self.fond=glob_dialog_bg
    
    self.procVnc=False
    self.etatVNC=False
    self.initWidgets()
    
    self.tickAnimCount=0
    self.tickAnim()
    self.majTick()
    


  def initWidgets(self):
    Frame.__init__(self, self.fenetre, padx=glob_dialog_padx, pady=glob_dialog_pady, bg=self.fond)
    self.pack(fill=BOTH)
    
    self.colonnes=Frame(self, bg=self.fond)
    self.colonnes.pack()
    
    self.txtFrame=Frame(self.colonnes, bg=self.fond, width=460)
    #self.txtFrame.pack_propagate(False)
    self.txtFrame.pack(side=LEFT, fill=BOTH, ipadx=10)
    
    self.titre = Label(self.txtFrame, text="Assistance à distance Afrikalan", bg=self.fond, font='Helvetica 20 bold', pady=8)
    self.titre.pack()

    self.messageEtat = Label(self.txtFrame, pady=4, bg=self.fond, font='Helvetica 14', justify="left")
    self.messageEtat.pack(anchor="w")

    self.linkStartStop=afrikalan.tkinter.Link(self.txtFrame, bg=self.fond, cursor="hand1", fg="#599de3", font='Helvetica 20', pady=16, command=self.evtLienStartStop)
    self.linkStartStop.pack(anchor="w")
    
    self.messageHint = Label(self.txtFrame, text=wrap("L'assistance à distance permet, lorsque vous êtes au téléphone avec un informaticien du projet Afrikalan, de partager votre écran et votre souris avec lui (vous deux pourrez voir l'écran et faire bouger la souris).", 60)+"\n\n"+wrap("Ce partage d'écran n'est possible que si vous l'autorisez en cliquant sur « Démarrer l’assistance à distance ». Le partage d'écran prendra fin dès que vous cliquerez sur « Arrêter l’assistance à distance »", 60), pady=2, bg=self.fond, justify="left", fg="#777777", font='Helvetica 12 italic')
    self.messageHint.pack(anchor="w")

    Frame(self.txtFrame, height=16, bg=self.fond).pack() #V-Separator

    self.icone_bouton_quitter=ImageTk.PhotoImage(file = r"/usr/share/afrikalan/images/transfert/icone_bouton_fermer.png")
    self.bouton_quitter = Button(self.txtFrame, text="Annuler", image=self.icone_bouton_quitter, compound="left", command=self.quit, bg=glob_button_bg)
    self.bouton_quitter.pack(anchor="e")  


    Frame(self.colonnes, width=16, bg=self.fond).pack(side=LEFT) #H-Separator

    
    self.imgUSBPlug=ImageTk.PhotoImage(file = r"/usr/share/afrikalan/images/assistance-250.png")
    self.lblImgUSBPlug=Label(self.colonnes, image=self.imgUSBPlug, pady=16)
    self.lblImgUSBPlug.pack(side=RIGHT)
  
  def tickAnim(self):
    self.tickAnimCount+=1
    if self.etatVNC:
      if self.tickAnimCount%2==1:
        self.messageEtat.configure(text=wrap("L'assistance à distance est démarrée.", 54))
      else:
        self.messageEtat.configure(text="")
    self.after(600, self.tickAnim)


  def majTick(self):
    self.etatVNC=self.verifEtatVNC()
    self.majAff()
    self.after(1000, self.majTick)
  
  def majAff(self):
    if self.etatVNC is True:
      #ANIMATED #self.messageEtat.configure(text=wrap("L'assistance à distance est démarrée.", 54)) 
      self.linkStartStop.configure(text=wrap("Arrêter l'assistance à distance.", 54))
    else:
      self.messageEtat.configure(text=wrap("L'assistance à distance est actuellement arrêtée.", 54))
      self.linkStartStop.configure(text=wrap("Démarrer l'assistance à distance.", 54))
  
  def evtVncFinished(self, retCode):
    self.procVnc=False
    self.etatVNC=False
    self.majAff()
    if retCode==-9 or retCode==0:
      message="L'assistance à distance est maintenant arrêtée."
    else:
      message="L'assistance à distance est maintenant arrêtée avec le code d'erreur "+str(retCode)+"."
    tkinter.messagebox.showinfo(title="Fin de l'assistance à distance", message=message)
  
  def verifEtatVNC(self):
    if self.procVnc is False:
      return False
    
    retCode=self.procVnc.poll()
    if retCode is not None:
      self.evtVncFinished(retCode)
      return False
    else:
      return True
  
  def demarreVNC(self):
    #X11VNC_REVERSE_CONNECTION_NO_AUTH=1
    self.procVnc=subprocess.Popen(["x11vnc", "-connect", "vnc-in.afrikalan.org", "-forever", "-loop", "-env", "X11VNC_REVERSE_CONNECTION_NO_AUTH=1"]) #"-ssl"ou "-stunnel", "TMP" # -solid pour désactiver bureau

  def stoppeVNC(self):
    for child in psutil.Process(self.procVnc.pid).children(recursive=True):
      child.kill()
    self.procVnc.kill()
  
  def evtExit(self):
    if self.etatVNC is not False:
      self.stoppeVNC()
  
  def evtLienStartStop(self): 
    if self.verifEtatVNC()!=self.etatVNC: #Action déjà traitée: STOP
      return False
    
    if self.etatVNC is False:
      self.demarreVNC()
    else:
      self.stoppeVNC()
    
    return True
    
    
    


##
## 
##
def parseArgs():
  argparser=argparse.ArgumentParser(description="Attend la connexion pour lancer l'application et affichant un message indiquant à l'utilisateur qu'une connexion est attendue et proposant de l'aide pour établir la connexion")
  argparser.add_argument("--cmdDesc", help="Identification textuelle de l'application qui sera lancée (pour la mentionner sur le message affiché à l'utilisateur", default='une application')
  argparser.add_argument('commande', nargs=argparse.REMAINDER, help="commande à lancer")
  return(argparser.parse_args())


class RunApp:
  def __init__(self, args):
    self.args=args
  
    root=Tk()
    root.title('Assitance à distance Afrikalan...')
  
    self.ifAssist=Interface_Assistance(root)
    afrikalan.tkinter.center(root)
    self.ifAssist.mainloop()
    print("exited ML ifAssist")
    self.ifAssist.destroy()
    self.ifAssist.evtExit()


args=parseArgs()
RunApp(args)
