#!/bin/bash
# Part of raspi-config http://github.com/asb/raspi-config
#
# See LICENSE file for copyright and license details

rm -rvf /tmp/pi/*

INTERACTIVE=True
ASK_TO_REBOOT=0

calc_wt_size() {
  # NOTE: it's tempting to redirect stderr to /dev/null, so supress error 
  # output from tput. However in this case, tput detects neither stdout or 
  # stderr is a tty and so only gives default 80, 24 values
  WT_HEIGHT=17
  WT_WIDTH=$(tput cols)

  if [ -z "$WT_WIDTH" ] || [ "$WT_WIDTH" -lt 60 ]; then
    WT_WIDTH=80
  fi
  if [ "$WT_WIDTH" -gt 178 ]; then
    WT_WIDTH=120
  fi
  WT_MENU_HEIGHT=$(($WT_HEIGHT-8))
}

set_config_var() {
  lua - "$1" "$2" "$3" <<EOF > "$3.bak"
local key=assert(arg[1])
local value=assert(arg[2])
local fn=assert(arg[3])
local file=assert(io.open(fn))
local made_change=false
for line in file:lines() do
  if line:match("^#?%s*"..key.."=.*$") then
    line=key.."="..value
    made_change=true
  end
  print(line)
end

if not made_change then
  print(key.."="..value)
end
EOF
mv "$3.bak" "$3"
}

get_config_var() {
  lua - "$1" "$2" <<EOF
local key=assert(arg[1])
local fn=assert(arg[2])
local file=assert(io.open(fn))
for line in file:lines() do
  local val = line:match("^#?%s*"..key.."=(.*)$")
  if (val ~= nil) then
    print(val)
    break
  end
end
EOF
}

# $1 is 0 to disable overscan, 1 to disable it
set_overscan() {
  # Stop if /boot is not a mountpoint
  if ! mountpoint -q /boot; then
    return 1
  fi

  [ -e /boot/config.txt ] || touch /boot/config.txt

  if [ "$1" -eq 0 ]; then # disable overscan
    sed /boot/config.txt -i -e "s/^overscan_/#overscan_/"
    set_config_var disable_overscan 1 /boot/config.txt
  else # enable overscan
    set_config_var disable_overscan 0 /boot/config.txt
  fi
}

attenteImagick () {

wmctrl -l | grep "ImageMagick"
while [ $? -ne 0 ]
do
sleep 1s
wmctrl -l | grep "ImageMagick"
done
}


do_overscan() {
#  whiptail --yesno "What would you like to do with overscan" 20 60 2 \
#    --yes-button Disable --no-button Enable
#  RET=$?
#  if [ $RET -eq 0 ] || [ $RET -eq 1 ]; then
#    ASK_TO_REBOOT=1
#    set_overscan $RET;
#  else
#    return 1
#  fi

actGauche=$(get_config_var overscan_left /boot/config.txt)
actDroite=$(get_config_var overscan_right /boot/config.txt)
actHaut=$(get_config_var overscan_top /boot/config.txt)
actBas=$(get_config_var overscan_bottom /boot/config.txt)

while true; do

killall display

nouvVals=$(yad --form --title "Positionnement de l'image (overscan)" --text "Cet outil vous permet de mieux positionner l'image sur l'écran, en jouant sur les marges. Il vous sera utile si une partie de l'image n'apparaît pas à l'écran" --field "Marge gauche":NUM " $actGauche\!-32..32" --field "Marge droite":NUM " $actDroite\!-32..32" --field "Marge haut":NUM " $actHaut\!-32..32" --field "Marge Bas":NUM " $actBas\!-32..32")
if [ $? -ne 0 ]; then
exit
fi

read nouvGauche nouvDroite nouvHaut nouvBas <<< $(echo $nouvVals | sed "s/,0*//g; s/|/ /g")

. afrikalanLireResol.sh
simPosGauche=$(echo $nouvGauche - $actGauche | bc)
simPosHaut=$(echo $nouvHaut - $actHaut | bc)
simTailleX=$(echo $RESX + $actGauche + $actDroite - $nouvGauche - $nouvDroite -4 |bc)
simTailleY=$(echo $RESY + $actBas + $actHaut - $nouvBas - $nouvHaut -4 |bc)
# -8

display -resize $simTailleX"x"$simTailleY\! -border 2x2 -bordercolor "#00FF00" /usr/share/afrikalan/images/fond.png&
attenteImagick
wmctrl -r "ImageMagick" -e "0,"$(echo $simPosGauche - 2 | bc)","$(echo $simPosHaut - 24 | bc)","$(echo $simTailleX + 4 | bc)","$(echo $simTailleY + 4 | bc)


yad --text "cliquez sur valider si le cadre vert est collé aux bords de l'écran. Si ce n'est pas le cas cliquez sur Annuler et affinez les réglages"
if [ $? -eq 0 ]; then
break
fi

done

set_config_var disable_overscan 0 /boot/config.txt
set_config_var overscan_left $nouvGauche /boot/config.txt
set_config_var overscan_right $nouvDroite /boot/config.txt
set_config_var overscan_top $nouvHaut /boot/config.txt
set_config_var overscan_bottom $nouvBas /boot/config.txt

mount / -oremount,rw
zl=$(echo "scale=2; $simTailleX / 820" | bc | sed "s/^[.]/0./g;")
mv /ro/home/pi/.config/midori/config /tmp/midori.config
cat /tmp/midori.config | grep -v zoom-level > /ro/home/pi/.config/midori/config
echo "zoom-level=$zl" >> /ro/home/pi/.config/midori/config

reboot



}

do_configure_keyboard() {
  dpkg-reconfigure keyboard-configuration &&
  printf "Reloading keymap. This may take a short while\n" &&
  invoke-rc.d keyboard-setup start
}

do_change_locale() {
  dpkg-reconfigure locales
}

do_change_timezone() {
  dpkg-reconfigure tzdata
}

do_memory_split() { # Memory Split
  if [ -e /boot/start_cd.elf ]; then
    # New-style memory split setting
    if ! mountpoint -q /boot; then
      return 1
    fi
    ## get current memory split from /boot/config.txt
    CUR_GPU_MEM=$(get_config_var gpu_mem /boot/config.txt)
    [ -z "$CUR_GPU_MEM" ] && CUR_GPU_MEM=64
    ## ask users what gpu_mem they want
    NEW_GPU_MEM=$(whiptail --inputbox "How much memory should the GPU have?  e.g. 16/32/64/128/256" \
      20 70 -- "$CUR_GPU_MEM" 3>&1 1>&2 2>&3)
    if [ $? -eq 0 ]; then
      set_config_var gpu_mem "$NEW_GPU_MEM" /boot/config.txt
      ASK_TO_REBOOT=1
    fi
  else # Old firmware so do start.elf renaming
    get_current_memory_split
    MEMSPLIT=$(whiptail --menu "Set memory split.\n$MEMSPLIT_DESCRIPTION" 20 60 10 \
      "240" "240MiB for ARM, 16MiB for VideoCore" \
      "224" "224MiB for ARM, 32MiB for VideoCore" \
      "192" "192MiB for ARM, 64MiB for VideoCore" \
      "128" "128MiB for ARM, 128MiB for VideoCore" \
      3>&1 1>&2 2>&3)
    if [ $? -eq 0 ]; then
      set_memory_split ${MEMSPLIT}
      ASK_TO_REBOOT=1
    fi
  fi
}

get_current_memory_split() {
  # Stop if /boot is not a mountpoint
  if ! mountpoint -q /boot; then
    return 1
  fi
  AVAILABLE_SPLITS="128 192 224 240"
  MEMSPLIT_DESCRIPTION=""
  for SPLIT in $AVAILABLE_SPLITS;do
    if [ -e /boot/arm${SPLIT}_start.elf ] && cmp /boot/arm${SPLIT}_start.elf /boot/start.elf >/dev/null 2>&1;then
      CURRENT_MEMSPLIT=$SPLIT
      MEMSPLIT_DESCRIPTION="Current: ${CURRENT_MEMSPLIT}MiB for ARM, $((256 - $CURRENT_MEMSPLIT))MiB for VideoCore"
      break
    fi
  done
}

set_memory_split() {
  cp -a /boot/arm${1}_start.elf /boot/start.elf
  sync
}

do_overclock() {

#  OVERCLOCK=$(whiptail --menu "Chose overclock preset" 20 60 10 \
#    "None" "700MHz ARM, 250MHz core, 400MHz SDRAM, 0 overvolt" \
#    "Modest" "800MHz ARM, 250MHz core, 400MHz SDRAM, 0 overvolt" \
#    "Medium" "900MHz ARM, 250MHz core, 450MHz SDRAM, 2 overvolt" \
#    "High" "950MHz ARM, 250MHz core, 450MHz SDRAM, 6 overvolt" \
#    "Turbo" "1000MHz ARM, 500MHz core, 600MHz SDRAM, 6 overvolt" \
#    3>&1 1>&2 2>&3)

killall yad
    case $1 in
      None)
        set_overclock None 700 250 400 0
        ;;
      Modest)
        set_overclock Modest 800 250 400 0
        ;;
      Medium)
        set_overclock Medium 900 250 450 2
        ;;
      High)
        set_overclock High 950 250 450 6
        ;;
      Turbo)
        set_overclock Turbo 1000 500 600 6
        ;;
      *)
        yad --width 600 --form --text "Overclocker Afrikalan permet de faire fonctionner ses composants électroniques au dessus de leur vitesse normale. Cela permet d'augmenter les performances. Attention toutefois: Un overclock trop important peut réduire la durée de vie des composants électroniques ou entrainer une instabilité du système. (redémarrages intempestifs)\n\nL'overclock est intéressant mais devrait se cantonner à des valeurs raisonables. 800MHz, et 900MHz sont de bons choix". --columns 1 --field="700MHz: Pas d'overclock (choix de sécurité)":BTN "$0 overclock None" --field="800MHz: Overclock léger":BTN "$0 overclock Modest" --field="900MHz: Overclock moyen (choix recommandé)":BTN "$0 overclock Medium" --field="950MHz: Overclock important (*Attention*)":BTN "$0 overclock High" --field="1GHz: Overclock maximum (*** ATTENTION ***)":BTN "$0 overclock Turbo" --button "gtk-cancel"
        ;;
    esac
    ASK_TO_REBOOT=1
}

set_overclock() {
  set_config_var arm_freq $2 /boot/config.txt &&
  set_config_var core_freq $3 /boot/config.txt &&
  set_config_var sdram_freq $4 /boot/config.txt &&
  set_config_var over_voltage $5 /boot/config.txt &&
  sudo mount / -oremount,rw
  sudo mount --bind /ro/var /var
  # now set up an init.d script
cat <<\EOF > /etc/init.d/switch_cpu_governor &&
#!/bin/sh
### BEGIN INIT INFO
# Provides:          switch_cpu_governor
# Required-Start: udev mountkernfs $remote_fs
# Required-Stop:
# Default-Start: S
# Default-Stop:
# Short-Description: Switch to ondemand cpu governor (unless shift key is pressed)
# Description:
### END INIT INFO

. /lib/lsb/init-functions

case "$1" in
  start)
    log_daemon_msg "Checking if shift key is held down"
    timeout 1 thd --dump /dev/input/event* | grep -q "LEFTSHIFT\|RIGHTSHIFT"
    if [ $? -eq 0 ]; then
      printf " Yes. Not switching scaling governor"
      log_end_msg 0
    else
      SYS_CPUFREQ_GOVERNOR=/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor
      [ -e $SYS_CPUFREQ_GOVERNOR ] && echo "ondemand" > $SYS_CPUFREQ_GOVERNOR
      echo 70 > /sys/devices/system/cpu/cpufreq/ondemand/up_threshold
      printf " No. Switching to ondemand scaling governor"
      log_end_msg 0
    fi
    ;;
  *)
    echo "Usage: $0 start" >&2
    exit 3
    ;;
esac
EOF
  chmod +x /etc/init.d/switch_cpu_governor &&
  update-rc.d switch_cpu_governor defaults &&
  yad --title "redémarrage" --text "votre choix a été enregistré, Afrikalan va redémarrer" --button "Redémarrer"
  reboot
}

disable_raspi_config_at_boot() {
  if [ -e /etc/profile.d/raspi-config.sh ]; then
    rm -f /etc/profile.d/raspi-config.sh
    sed -i /etc/inittab \
      -e "s/^#\(.*\)#\s*RPICFG_TO_ENABLE\s*/\1/" \
      -e "/#\s*RPICFG_TO_DISABLE/d"
    telinit q
  fi
}

do_update() {
  apt-get update &&
  apt-get install raspi-config &&
  printf "Sleeping 5 seconds before reloading raspi-config\n" &&
  sleep 5 &&
  exec raspi-config
}

do_audio() {
  AUDIO_OUT=$(whiptail --menu "Choose the audio output" 20 60 10 \
    "0" "Auto" \
    "1" "Force 3.5mm ('headphone') jack" \
    "2" "Force HDMI" \
    3>&1 1>&2 2>&3)
  if [ $? -eq 0 ]; then
    amixer cset numid=3 "$AUDIO_OUT"
  fi
}

do_finish() {
  disable_raspi_config_at_boot
  if [ $ASK_TO_REBOOT -eq 1 ]; then
    whiptail --yesno "Would you like to reboot now?" 20 60 2
    if [ $? -eq 0 ]; then # yes
      sync
      reboot
    fi
  fi
  exit 0
}

# $1 = filename, $2 = key name
get_json_string_val() {
  sed -n -e "s/^[[:space:]]*\"$2\"[[:space:]]*:[[:space:]]*\"\(.*\)\"[[:space:]]*,$/\1/p" $1
}

do_apply_os_config() {
  [ -e /boot/os_config.json ] || return 0
  NOOBSFLAVOUR=$(get_json_string_val /boot/os_config.json flavour)
  NOOBSLANGUAGE=$(get_json_string_val /boot/os_config.json language)
  NOOBSKEYBOARD=$(get_json_string_val /boot/os_config.json keyboard)

  if [ -n "$NOOBSFLAVOUR" ]; then
    printf "Setting flavour to %s based on os_config.json from NOOBS. May take a while\n" "$NOOBSFLAVOUR"

    if printf "%s" "$NOOBSFLAVOUR" | grep -q "Scratch"; then
      disable_raspi_config_at_boot
      enable_boot_to_scratch
    else
      printf "Unrecognised flavour. Ignoring\n"
    fi
  fi

  # TODO: currently ignores en_gb settings as we assume we are running in a 
  # first boot context, where UK English settings are default
  case "$NOOBSLANGUAGE" in
    "en")
      if [ "$NOOBSKEYBOARD" = "gb" ]; then
        DEBLANGUAGE="" # UK english is the default, so ignore
      else
        DEBLANGUAGE="en_US.UTF-8"
      fi
      ;;
    "de")
      DEBLANGUAGE="de_DE.UTF-8"
      ;;
    "fi")
      DEBLANGUAGE="fi_FI.UTF-8"
      ;;
    "fr")
      DEBLANGUAGE="fr_FR.UTF-8"
      ;;
    "hu")
      DEBLANGUAGE="hu_HU.UTF-8"
      ;;
    "ja")
      DEBLANGUAGE="ja_JP.UTF-8"
      ;;
    "nl")
      DEBLANGUAGE="nl_NL.UTF-8"
      ;;
    "pt")
      DEBLANGUAGE="pt_PT.UTF-8"
      ;;
    "ru")
      DEBLANGUAGE="ru_RU.UTF-8"
      ;;
    "zh_CN")
      DEBLANGUAGE="zh_CN.UTF-8"
      ;;
    *)
      printf "Language '%s' not handled currently. Run sudo raspi-config to set up" "$NOOBSLANGUAGE"
      ;;
  esac

  if [ -n "$DEBLANGUAGE" ]; then
    printf "Setting language to %s based on os_config.json from NOOBS. May take a while\n" "$DEBLANGUAGE"
    cat << EOF | debconf-set-selections
locales   locales/locales_to_be_generated multiselect     $DEBLANGUAGE UTF-8
EOF
    rm /etc/locale.gen
    dpkg-reconfigure -f noninteractive locales
    update-locale LANG="$DEBLANGUAGE"
    cat << EOF | debconf-set-selections
locales   locales/default_environment_locale select       $DEBLANGUAGE
EOF
  fi

  if [ -n "$NOOBSKEYBOARD" -a "$NOOBSKEYBOARD" != "gb" ]; then
    printf "Setting keyboard layout to %s based on os_config.json from NOOBS. May take a while\n" "$NOOBSKEYBOARD"
    sed -i /etc/default/keyboard -e "s/^XKBLAYOUT.*/XKBLAYOUT=\"$NOOBSKEYBOARD\"/"
    dpkg-reconfigure -f noninteractive keyboard-configuration
    invoke-rc.d keyboard-setup start
  fi
  return 0
}

if [ $(id -u) -ne 0 ]; then
  yad --text "Vous devez etre root" --button "gtk-close"
  exit 1
fi


case $1 in
overclock) do_overclock $2 ;;
esac
case $1 in
overscan) do_overscan ;;
esac
