#!/usr/bin/python3

import afrikalan.web
import afrikalan.xwin

from lxml import etree
import os

import re

import argparse


class ContentHandler:
  def __init__(self, contentPath, callfrom):
    self.callfrom=callfrom
    self.contentPath=contentPath
    self.baseDir=os.path.split(self.contentPath)[0]
    self.xmlRoot=etree.parse(self.contentPath)
    self.initPagesId()

    self.winscanner=afrikalan.xwin.Scanner()
    self.winscanner.maxScanWinDepth=2
    

  def getImagesDim(self):
    defaultWidth=420
    defaultHeigth=420
    
    try:
      searchIs=self.xmlRoot.xpath('/man/layout/images-size')
      return([int(searchIs[0].attrib.get("width", defaultWidth)), int(searchIs[0].attrib.get("height", defaultHeigth))])
    except IndexError: 
      return([defaultWidth, defaultHeigth])
    
    

  def getTitle(self):
    searchTitle=self.xmlRoot.xpath('/man/title')
    try:
      return(searchTitle[0].text)
    except IndexError:
      return("")

    
  def getDescription(self):
    searchDesc=self.xmlRoot.xpath('/man/description')
    try:
      return(searchDesc[0].text)
    except IndexError:
      return("")

  def getMenuItemInfos(self):
    searchDesc=self.xmlRoot.xpath('/man/menuitem')
    try:
      iconPath=searchDesc[0].attrib.get('icon', False)
      if iconPath is not False:
        iconPath=self.getDependencyPath(iconPath)
      return({'visible': int(searchDesc[0].attrib.get('visible', '0')), 'order': int(searchDesc[0].attrib.get('order', '500')), 'icon': iconPath})
    except IndexError:
      return({})



  
  def getNextLinkText(self):
    searchNLT=self.xmlRoot.xpath('/man/layout/next-link-text')
    try:
      return(searchNLT[0].text)
    except IndexError:
      return("Étape suivante ->")

  def isWinStayOnTop(self):
    searchSOT=self.xmlRoot.xpath('/man/params/stayontop')
    try:
      return(int(searchSOT[0].attrib.get('value', '0')))
    except IndexError:
      return(0)

  def isQuitOnFocusOut(self):
    return (self.callfrom=="dock")


  def initPagesId(self):
    searchPages=self.xmlRoot.xpath('/man/pages/page')
    actid=0
    for xmlNodePage in searchPages:
      xmlNodePage.attrib['affman-autoid']=str(actid)
      actid+=1

  def getPages(self):
    searchPages=self.xmlRoot.xpath('/man/pages/page')
    arObjPages=[]

    for xmlNodePage in searchPages:
      actPage={}
      try:
        actPage['title']=xmlNodePage.xpath('title')[0].text
      except IndexError:
        pass
      
      instructions=[]
      for nodeInst in xmlNodePage.xpath('instruction'):
        instructions.append(nodeInst.text)
      actPage['instructions']=instructions

      hints=[]
      for nodeHint in xmlNodePage.xpath('hint'):
        hints.append(nodeHint.text)
      actPage['hints']=hints
        
      try:
        imgPath=xmlNodePage.xpath('img')[0].attrib.get("path")
      except IndexError:
        pass
      else:
        if imgPath is not None:
          actPage['image']=self.getDependencyPath(imgPath)
      
      arObjPages.append(actPage)
    return(arObjPages)

  def getAnims(self, pageId):
    return(self.xmlRoot.xpath("/man/pages/page[@affman-autoid='"+str(pageId)+"']/anim"))


  def getDependencyPath(self, path):
    if os.path.isabs(path):
      return(imgPath)
    else:
      return(os.path.abspath(os.path.normpath(os.path.join(self.baseDir, path))))
  
  def findStartPage(self):
    startPageId=0 #default return value    

    nodesMatchCallFrom=self.xmlRoot.xpath("/man/pages/page/matchselect[@type='call-from']")
    for nodeMatchCallFrom in nodesMatchCallFrom:
      if nodeMatchCallFrom.text==self.callfrom:
        startPageId=int(nodeMatchCallFrom.getparent().attrib.get('affman-autoid'))

    
    if self.callfrom=="dock": #follwing methods doesn't have sense if the app isn't ran form the dock

      nodesMatchDef=self.xmlRoot.xpath("/man/pages/page/matchselect[@type='default']")
      for nodeMatchDef in nodesMatchDef:
        startPageId=int(nodeMatchDef.getparent().attrib.get('affman-autoid'))


      nodesMatchUrl=self.xmlRoot.xpath("/man/pages/page/matchselect[@type='viewedurl']")
      if len(nodesMatchUrl)>0:
        url=afrikalan.web.getViewedURL()
  
        for nodeMatchUrl in nodesMatchUrl:
          if re.match(nodeMatchUrl.text, url):
            startPageId=int(nodeMatchUrl.getparent().attrib.get('affman-autoid'))
            
      
      nodesMatchWin=self.xmlRoot.xpath("/man/pages/page/matchselect[@type='wintitle']")
      if len(nodesMatchWin)>0:
        self.winscanner.scanWins()
        for nodeMatchWin in nodesMatchWin:
          print(nodeMatchWin.text)
          reTitle=re.compile(nodeMatchWin.text)
          for title in self.winscanner.winTitleList:
            if reTitle.match(title):
              startPageId=int(nodeMatchWin.getparent().attrib.get('affman-autoid'))

    return(startPageId);
