#!/bin/python3

import tkinter
import tkinter.font
import tkinter.ttk

from PIL import Image, ImageTk
import textwrap


glob_dialog_bg="#F8F8F8"
glob_dialog_padx=16
glob_dialog_pady=10
glob_infoframe_bg="#FFFFFF"
glob_button_bg="#E0E0E0" #was: #D4D4D4

class Interface_Liste (tkinter.Frame):
  
  def __init__(self, fenetre, arItems, txtMessage=False, scrollFrame=True):
    self.choice=False
    self.fenetre=fenetre
    self.fond=glob_dialog_bg
    self.arItems=arItems
    
    tkinter.Frame.__init__(self, fenetre, padx=glob_dialog_padx, pady=glob_dialog_pady, bg=self.fond)
    
    self.pack(fill=tkinter.BOTH)
    
    if txtMessage is not False:
      self.message = tkinter.Label(self, text=txtMessage, pady=18, bg=self.fond, font='Helvetica 16')
      self.message.grid(row=0, columnspan=5)
    
    if scrollFrame:
      self.frmChoices=ScrollableFrame(self, width=439, height=500)
      self.genChoicesWidgets(self.frmChoices.scrollable_frame)
    else:
      self.frmChoices=tkinter.Frame(self, width=False, height=False, bg=self.fond)
      self.genChoicesWidgets(self.frmChoices)
    self.frmChoices.grid(row=1, column=0, columnspan=5)

    self.icone_bouton_quitter=ImageTk.PhotoImage(file = r"/usr/share/afrikalan/images/transfert/icone_bouton_fermer.png")
    self.bouton_quitter = tkinter.Button(self, text="Annuler", image=self.icone_bouton_quitter, compound="left", command=self.quit, bg=glob_button_bg)
    self.bouton_quitter.grid(row=3, column=4, sticky="E", pady=6)  

  def genChoicesWidgets(self, container):
    actrow=0
    
    for infositem in self.arItems:
      item=ItemListe(container, infositem, self.evtItemSelected)
      item.grid(row=actrow, sticky="we")
      tkinter.Frame(container, height=12, bg=self.fond).grid(row=actrow+1, column=0) #H-Separator
      actrow+=2

    
  #Event: Target bluetooth host selected by user
  def evtItemSelected(self, flavourId):
    self.choice=flavourId
    self.quit()


class ItemListe(tkinter.Frame):
  def __init__(self, container, infositem, cbSelect):
    self.infositem=infositem
    self.cbSelect=cbSelect
    self.fond=glob_dialog_bg
    self.fondHightlight="#cbe3e6"
    
    tkinter.Frame.__init__(self, container, padx=glob_dialog_padx, pady=glob_dialog_pady, bg=self.fond)
    self.bind("<Button-1>", self.evtChoosen)
    
    self.icone=ImageTk.PhotoImage(file = self.infositem.get('icon'))
    self.lblIcone = tkinter.Label(self, image=self.icone, bg=self.fond)
    self.lblIcone.grid(row=0, column=0, rowspan=2, sticky="w" )
    self.lblIcone.bind("<Button-1>", self.evtChoosen)

    self.sep=tkinter.Frame(self, width=16, bg=self.fond)
    self.sep.grid(row=0, column=1) #H-Separator

    arlines=textwrap.wrap(self.infositem.get('title'), width=38)
    self.lblName=tkinter.Label(self, text="\n".join(arlines), justify="left", bg=self.fond, fg="#555555", font="helvetica 14")
    self.lblName.bind("<Button-1>", self.evtChoosen)
    self.lblName.grid(row=0, column=2, sticky="w")
      
    self.bind("<Enter>", self.hiLight)
    self.bind("<Leave>", self.unHiLight)

      
    #btnChoose=tkinter.Button(self, text="Choisir", command=evtChoosen, bg=glob_button_bg)
    #btnChoose.grid(row=0, rowspan=2, column=2)
    
    self.lblDescr=False
    if self.infositem.get('description') is not False:
      arlines=textwrap.wrap(self.infositem.get('description'), width=50)

      self.lblDescr=tkinter.Label(self, text="\n".join(arlines), justify="left", bg=self.fond)
      fontDesc=tkinter.font.Font(self.lblDescr, self.lblDescr.cget("font"))
      fontDesc.configure(slant="italic")
      self.lblDescr.configure(font=fontDesc)
      self.lblDescr.bind("<Button-1>", self.evtChoosen)
      self.lblDescr.grid(row=1, column=2, sticky="w")
      
  def hiLight(self, evt):
    self.configure(bg=self.fondHightlight)
    self.lblIcone.configure(bg=self.fondHightlight)
    self.lblName.configure(bg=self.fondHightlight)
    if self.lblDescr is not False:
      self.lblDescr.configure(bg=self.fondHightlight)
    self.sep.configure(bg=self.fondHightlight)
    
  def unHiLight(self, evt):
    self.configure(bg=self.fond)
    self.lblIcone.configure(bg=self.fond)
    self.lblName.configure(bg=self.fond)
    if self.lblDescr is not False:
      self.lblDescr.configure(bg=self.fond)
    self.sep.configure(bg=self.fond)


  def evtChoosen(self, evt=False):
    self.cbSelect(self.infositem.get('value'))



class ScrollableFrame(tkinter.Frame):
  def __init__(self, container, width=600, height=500, *args, **kwargs):
    super().__init__(container, *args, **kwargs)
    self.canvas = tkinter.Canvas(self, width=width, height=height, background=glob_dialog_bg, borderwidth=0)
    scrollbar = tkinter.ttk.Scrollbar(self, orient="vertical", command=self.canvas.yview)
    self.scrollable_frame = tkinter.Frame(self.canvas, bg=glob_dialog_bg, width=width, height=height, bd=0)
    
    self.scrollStep=1

    self.scrollable_frame.bind(
        "<Configure>",
        lambda e: self.canvas.configure(
            scrollregion=self.canvas.bbox("all")
        )
    )

    self.canvas.create_window((0, 0), window=self.scrollable_frame, anchor="nw")
    #root.bind_all("<MouseWheel>", self._on_mousewheel)
    self.canvas.bind_all("<Button-4>", self.scrollUp)
    self.canvas.bind_all("<Button-5>", self.scrollDown)
    self.canvas.configure(yscrollcommand=scrollbar.set)

    self.canvas.pack(side="left", fill="both", expand=True)
    scrollbar.pack(side="right", fill="y")

    return


  #def _on_mousewheel(self, event):
    #self.canvas.yview_scroll(int(-1*(event.delta/120)), "units")
  def scrollUp(self, event):
    self.canvas.yview_scroll(self.scrollStep*-1, "units")
  def scrollDown(self, event):
    self.canvas.yview_scroll(self.scrollStep, "units")
        

def center(toplevel):
    toplevel.update_idletasks()

    # Tkinter way to find the screen resolution
    screen_width = toplevel.winfo_screenwidth()
    screen_height = toplevel.winfo_screenheight()

    size = tuple(int(_) for _ in toplevel.geometry().split('+')[0].split('x'))
    x = screen_width/2 - size[0]/2
    y = screen_height/2 - size[1]/2

    toplevel.geometry("+%d+%d" % (x, y))



class Link(tkinter.Label):
  def __init__(self, container, command=None, *args, **kwargs):
    self.command=command
    super().__init__(container, *args, **kwargs)
    
    self.configure(activeforeground="#FF0000")
    
    self.fontLink=tkinter.font.Font(self, self.cget("font"))
    self.fontLinkUnderlined=tkinter.font.Font(self, self.cget("font"))
    self.fontLinkUnderlined.configure(underline=True)
    self.bind("<Enter>", lambda e : e.widget.configure(font=self.fontLinkUnderlined) if e.widget['state']!='disabled' else False )
    self.bind("<Leave>", lambda e : e.widget.configure(font=self.fontLink))
    self.bind("<Button-1>", self.evtClick) 
    self.bind("<ButtonRelease-1>", self.evtClickRelease) 
    self.preventDblClick=True
    self.justClicked=False
    self.rearmClickDelay=1000
    self.colorInact=self.cget("foreground")
    self.colorAct=deltaColorStr(self.colorInact, 60)
  
  
  def rearmClick(self):
    self.justClicked=False
    
  
  def evtClick(self, evt):
    self.configure(fg=self.colorAct)
    if self['state']=='disabled' or self.justClicked:
      return(False)
    self.justClicked=True
    self.after(self.rearmClickDelay, self.rearmClick)
    self.command()
  
  def evtClickRelease(self, evt):
    self.configure(fg=self.colorInact)
    
  
def deltaColorStr(colorStr, delta):
  nvColStr="#"
  for pos in [1, 3, 5]:
    valComposante=int(colorStr[pos:pos+2], 16)
    valComposante+=delta
    valComposante=max(valComposante, 0)
    valComposante=min(valComposante, 255)
    nvColStr+=format(valComposante, 'x')
  return(nvColStr)
