#!/usr/bin/python3

from lxml import etree
import os
import subprocess
import afrikalan.config

afklConf=afrikalan.config.get(["main"])

class Handler:
  
  def __init__(self, handlerPath, handlerArgs=[]):
    self.handlerPath=handlerPath
    self.handlerArgs=handlerArgs
    self.xmlRoot=self.getXmlRoot()

  def getType(self):
      f, ext=os.path.splitext(self.handlerPath)
      if ext==".xml":
        return("static")
      else:
        return("dynamic")
      
      
  def getXmlRoot(self):
    handlerType=self.getType()
    if handlerType=="dynamic":
      xml=subprocess.check_output([self.handlerPath, "listFiles"] + self.handlerArgs)
      return(etree.fromstring(xml))
    else:
      return(etree.parse(self.handlerPath))
    


  def getMeta(self, xpath, default):
    ret=default
    nodesName=self.xmlRoot.xpath(xpath)
    if len(nodesName)!=0:
      ret=nodesName[0].text
    return(ret)

  def appName(self, default=""):
    return(self.getMeta('/transfer/eltname', default=default))

  def appDecr(self, default=""):
    return(self.getMeta('/transfer/description', default=default))

  def appIcon(self, default=False):
    return(self.getMeta('/transfer/icon', default=default))

  def appIconAbs(self):
    iconPath=self.appIcon()
    if iconPath is False:
      return(False)
    
    if not os.path.isabs(iconPath):
      basePath, handlerFName=os.path.split(self.handlerPath)
      iconPath=os.path.normpath(os.path.join(basePath, iconPath))
      
    return(iconPath)


  def availableTransferModes(self):
    ret={'usb': 1, 'bluetooth': 1, 'wifi': 0}
    nodesModes=self.xmlRoot.xpath('/transfer/modes')
    if len(nodesModes)!=0:
      if nodesModes[0].attrib.get('usb')=='0':
        ret['usb']=0
      if nodesModes[0].attrib.get('bluetooth')=='0':
        ret['bluetooth']=0
      if nodesModes[0].attrib.get('wifi')=='1':
        ret['wifi']=1

    if afklConf.get("afrikalan_transfert_nobluetooth", "0")=='1':
      ret['bluetooth']=0

    return(ret)


  def order(self):
    nodesPlacement=self.xmlRoot.xpath('/transfer/placement')
    if len(nodesPlacement)!=0:
      order=nodesPlacement[0].attrib.get('order', '500')
    else:
      order='500'
    return(int(order))


  def checkOk(self):
    files=self.xmlRoot.xpath('/transfer/files/file[@check-exists-needed=\'1\']')
    for actfile in files:
      path=actfile.get("path", False)
      if not os.path.isfile(path):
        return(False)
    
    return(True)
